# Generated from DSLSQL.g4 by ANTLR 4.7.1
# encoding: utf-8
from antlr4 import *
from io import StringIO
from typing import TextIO
import sys

def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\63")
        buf.write("\u012e\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\3\2\3\2\3\2\7\2\60\n\2")
        buf.write("\f\2\16\2\63\13\2\3\3\3\3\3\3\3\3\3\3\5\3:\n\3\3\3\5\3")
        buf.write("=\n\3\3\3\7\3@\n\3\f\3\16\3C\13\3\3\3\3\3\3\3\3\3\3\3")
        buf.write("\3\3\3\3\3\3\7\3M\n\3\f\3\16\3P\13\3\3\3\3\3\3\3\3\3\3")
        buf.write("\3\3\3\5\3X\n\3\3\3\5\3[\n\3\3\3\7\3^\n\3\f\3\16\3a\13")
        buf.write("\3\3\3\3\3\5\3e\n\3\3\3\3\3\7\3i\n\3\f\3\16\3l\13\3\3")
        buf.write("\3\3\3\3\3\3\3\7\3r\n\3\f\3\16\3u\13\3\3\3\3\3\7\3y\n")
        buf.write("\3\f\3\16\3|\13\3\3\3\3\3\7\3\u0080\n\3\f\3\16\3\u0083")
        buf.write("\13\3\3\3\3\3\7\3\u0087\n\3\f\3\16\3\u008a\13\3\3\3\3")
        buf.write("\3\3\3\3\3\3\3\5\3\u0091\n\3\3\3\5\3\u0094\n\3\3\3\7\3")
        buf.write("\u0097\n\3\f\3\16\3\u009a\13\3\3\3\3\3\3\3\5\3\u009f\n")
        buf.write("\3\3\3\5\3\u00a2\n\3\3\3\7\3\u00a5\n\3\f\3\16\3\u00a8")
        buf.write("\13\3\3\3\3\3\5\3\u00ac\n\3\3\3\3\3\3\3\3\3\3\3\3\3\3")
        buf.write("\3\5\3\u00b5\n\3\3\3\5\3\u00b8\n\3\3\3\7\3\u00bb\n\3\f")
        buf.write("\3\16\3\u00be\13\3\3\3\3\3\3\3\3\3\3\3\3\3\3\3\5\3\u00c7")
        buf.write("\n\3\3\3\5\3\u00ca\n\3\3\3\7\3\u00cd\n\3\f\3\16\3\u00d0")
        buf.write("\13\3\3\3\3\3\3\3\3\3\3\3\5\3\u00d7\n\3\3\3\5\3\u00da")
        buf.write("\n\3\3\3\7\3\u00dd\n\3\f\3\16\3\u00e0\13\3\3\3\3\3\3\3")
        buf.write("\3\3\3\3\5\3\u00e7\n\3\3\3\5\3\u00ea\n\3\3\3\7\3\u00ed")
        buf.write("\n\3\f\3\16\3\u00f0\13\3\3\3\5\3\u00f3\n\3\3\4\3\4\3\5")
        buf.write("\3\5\3\6\3\6\3\7\3\7\3\b\3\b\3\b\3\t\3\t\3\t\3\t\3\n\3")
        buf.write("\n\3\13\3\13\3\f\3\f\5\f\u010a\n\f\3\r\3\r\3\r\3\r\5\r")
        buf.write("\u0110\n\r\3\16\3\16\3\17\3\17\5\17\u0116\n\17\3\20\3")
        buf.write("\20\3\21\3\21\3\22\3\22\3\23\3\23\3\23\7\23\u0121\n\23")
        buf.write("\f\23\16\23\u0124\13\23\3\24\3\24\3\25\3\25\5\25\u012a")
        buf.write("\n\25\3\26\3\26\3\26\2\2\27\2\4\6\b\n\f\16\20\22\24\26")
        buf.write("\30\32\34\36 \"$&(*\2\21\3\2\3\4\3\2\6\7\3\2\t\n\3\2\f")
        buf.write("\r\3\2\16\16\3\2\17\20\3\2\21\22\3\2\23\24\3\2\25\26\3")
        buf.write("\2\27\30\3\2\32\33\3\2\34\37\3\2 !\3\2\"#\3\2$%\2\u0150")
        buf.write("\2\61\3\2\2\2\4\u00f2\3\2\2\2\6\u00f4\3\2\2\2\b\u00f6")
        buf.write("\3\2\2\2\n\u00f8\3\2\2\2\f\u00fa\3\2\2\2\16\u00fc\3\2")
        buf.write("\2\2\20\u00ff\3\2\2\2\22\u0103\3\2\2\2\24\u0105\3\2\2")
        buf.write("\2\26\u0109\3\2\2\2\30\u010f\3\2\2\2\32\u0111\3\2\2\2")
        buf.write("\34\u0115\3\2\2\2\36\u0117\3\2\2\2 \u0119\3\2\2\2\"\u011b")
        buf.write("\3\2\2\2$\u011d\3\2\2\2&\u0125\3\2\2\2(\u0129\3\2\2\2")
        buf.write("*\u012b\3\2\2\2,-\5\4\3\2-.\5\22\n\2.\60\3\2\2\2/,\3\2")
        buf.write("\2\2\60\63\3\2\2\2\61/\3\2\2\2\61\62\3\2\2\2\62\3\3\2")
        buf.write("\2\2\63\61\3\2\2\2\64\65\t\2\2\2\65\66\5\24\13\2\66\67")
        buf.write("\7\5\2\2\679\5\26\f\28:\t\3\2\298\3\2\2\29:\3\2\2\2:<")
        buf.write("\3\2\2\2;=\5\20\t\2<;\3\2\2\2<=\3\2\2\2=A\3\2\2\2>@\5")
        buf.write("\16\b\2?>\3\2\2\2@C\3\2\2\2A?\3\2\2\2AB\3\2\2\2BD\3\2")
        buf.write("\2\2CA\3\2\2\2DE\7\b\2\2EF\5\36\20\2F\u00f3\3\2\2\2GN")
        buf.write("\t\4\2\2HM\5\6\4\2IM\5\b\5\2JM\5\n\6\2KM\5\f\7\2LH\3\2")
        buf.write("\2\2LI\3\2\2\2LJ\3\2\2\2LK\3\2\2\2MP\3\2\2\2NL\3\2\2\2")
        buf.write("NO\3\2\2\2OQ\3\2\2\2PN\3\2\2\2QR\5\36\20\2RS\7\b\2\2S")
        buf.write("T\5\24\13\2TU\7\5\2\2UW\5\26\f\2VX\t\3\2\2WV\3\2\2\2W")
        buf.write("X\3\2\2\2XZ\3\2\2\2Y[\5\20\t\2ZY\3\2\2\2Z[\3\2\2\2[_\3")
        buf.write("\2\2\2\\^\5\16\b\2]\\\3\2\2\2^a\3\2\2\2_]\3\2\2\2_`\3")
        buf.write("\2\2\2`d\3\2\2\2a_\3\2\2\2bc\7\13\2\2ce\5\"\22\2db\3\2")
        buf.write("\2\2de\3\2\2\2e\u00f3\3\2\2\2fj\t\5\2\2gi\n\6\2\2hg\3")
        buf.write("\2\2\2il\3\2\2\2jh\3\2\2\2jk\3\2\2\2km\3\2\2\2lj\3\2\2")
        buf.write("\2mn\7\b\2\2n\u00f3\5\36\20\2os\t\7\2\2pr\n\6\2\2qp\3")
        buf.write("\2\2\2ru\3\2\2\2sq\3\2\2\2st\3\2\2\2t\u00f3\3\2\2\2us")
        buf.write("\3\2\2\2vz\t\b\2\2wy\n\6\2\2xw\3\2\2\2y|\3\2\2\2zx\3\2")
        buf.write("\2\2z{\3\2\2\2{\u00f3\3\2\2\2|z\3\2\2\2}\u0081\t\t\2\2")
        buf.write("~\u0080\n\6\2\2\177~\3\2\2\2\u0080\u0083\3\2\2\2\u0081")
        buf.write("\177\3\2\2\2\u0081\u0082\3\2\2\2\u0082\u00f3\3\2\2\2\u0083")
        buf.write("\u0081\3\2\2\2\u0084\u0088\t\n\2\2\u0085\u0087\n\6\2\2")
        buf.write("\u0086\u0085\3\2\2\2\u0087\u008a\3\2\2\2\u0088\u0086\3")
        buf.write("\2\2\2\u0088\u0089\3\2\2\2\u0089\u00f3\3\2\2\2\u008a\u0088")
        buf.write("\3\2\2\2\u008b\u008c\t\13\2\2\u008c\u008d\5\32\16\2\u008d")
        buf.write("\u008e\7\31\2\2\u008e\u0090\5\30\r\2\u008f\u0091\t\3\2")
        buf.write("\2\u0090\u008f\3\2\2\2\u0090\u0091\3\2\2\2\u0091\u0093")
        buf.write("\3\2\2\2\u0092\u0094\5\20\t\2\u0093\u0092\3\2\2\2\u0093")
        buf.write("\u0094\3\2\2\2\u0094\u0098\3\2\2\2\u0095\u0097\5\16\b")
        buf.write("\2\u0096\u0095\3\2\2\2\u0097\u009a\3\2\2\2\u0098\u0096")
        buf.write("\3\2\2\2\u0098\u0099\3\2\2\2\u0099\u00f3\3\2\2\2\u009a")
        buf.write("\u0098\3\2\2\2\u009b\u009c\t\f\2\2\u009c\u009e\5\24\13")
        buf.write("\2\u009d\u009f\t\3\2\2\u009e\u009d\3\2\2\2\u009e\u009f")
        buf.write("\3\2\2\2\u009f\u00a1\3\2\2\2\u00a0\u00a2\5\20\t\2\u00a1")
        buf.write("\u00a0\3\2\2\2\u00a1\u00a2\3\2\2\2\u00a2\u00a6\3\2\2\2")
        buf.write("\u00a3\u00a5\5\16\b\2\u00a4\u00a3\3\2\2\2\u00a5\u00a8")
        buf.write("\3\2\2\2\u00a6\u00a4\3\2\2\2\u00a6\u00a7\3\2\2\2\u00a7")
        buf.write("\u00ab\3\2\2\2\u00a8\u00a6\3\2\2\2\u00a9\u00aa\7\b\2\2")
        buf.write("\u00aa\u00ac\5\34\17\2\u00ab\u00a9\3\2\2\2\u00ab\u00ac")
        buf.write("\3\2\2\2\u00ac\u00f3\3\2\2\2\u00ad\u00ae\t\r\2\2\u00ae")
        buf.write("\u00af\5\36\20\2\u00af\u00b0\7\b\2\2\u00b0\u00b1\5\24")
        buf.write("\13\2\u00b1\u00b2\7\5\2\2\u00b2\u00b4\5\26\f\2\u00b3\u00b5")
        buf.write("\t\3\2\2\u00b4\u00b3\3\2\2\2\u00b4\u00b5\3\2\2\2\u00b5")
        buf.write("\u00b7\3\2\2\2\u00b6\u00b8\5\20\t\2\u00b7\u00b6\3\2\2")
        buf.write("\2\u00b7\u00b8\3\2\2\2\u00b8\u00bc\3\2\2\2\u00b9\u00bb")
        buf.write("\5\16\b\2\u00ba\u00b9\3\2\2\2\u00bb\u00be\3\2\2\2\u00bc")
        buf.write("\u00ba\3\2\2\2\u00bc\u00bd\3\2\2\2\u00bd\u00f3\3\2\2\2")
        buf.write("\u00be\u00bc\3\2\2\2\u00bf\u00c0\t\16\2\2\u00c0\u00c1")
        buf.write("\5\24\13\2\u00c1\u00c2\7\5\2\2\u00c2\u00c3\5\26\f\2\u00c3")
        buf.write("\u00c4\7\b\2\2\u00c4\u00c6\5 \21\2\u00c5\u00c7\t\3\2\2")
        buf.write("\u00c6\u00c5\3\2\2\2\u00c6\u00c7\3\2\2\2\u00c7\u00c9\3")
        buf.write("\2\2\2\u00c8\u00ca\5\20\t\2\u00c9\u00c8\3\2\2\2\u00c9")
        buf.write("\u00ca\3\2\2\2\u00ca\u00ce\3\2\2\2\u00cb\u00cd\5\16\b")
        buf.write("\2\u00cc\u00cb\3\2\2\2\u00cd\u00d0\3\2\2\2\u00ce\u00cc")
        buf.write("\3\2\2\2\u00ce\u00cf\3\2\2\2\u00cf\u00f3\3\2\2\2\u00d0")
        buf.write("\u00ce\3\2\2\2\u00d1\u00d2\t\17\2\2\u00d2\u00d3\5\24\13")
        buf.write("\2\u00d3\u00d4\7\5\2\2\u00d4\u00d6\5\26\f\2\u00d5\u00d7")
        buf.write("\t\3\2\2\u00d6\u00d5\3\2\2\2\u00d6\u00d7\3\2\2\2\u00d7")
        buf.write("\u00d9\3\2\2\2\u00d8\u00da\5\20\t\2\u00d9\u00d8\3\2\2")
        buf.write("\2\u00d9\u00da\3\2\2\2\u00da\u00de\3\2\2\2\u00db\u00dd")
        buf.write("\5\16\b\2\u00dc\u00db\3\2\2\2\u00dd\u00e0\3\2\2\2\u00de")
        buf.write("\u00dc\3\2\2\2\u00de\u00df\3\2\2\2\u00df\u00f3\3\2\2\2")
        buf.write("\u00e0\u00de\3\2\2\2\u00e1\u00e2\t\20\2\2\u00e2\u00e3")
        buf.write("\5\24\13\2\u00e3\u00e4\7\5\2\2\u00e4\u00e6\5\26\f\2\u00e5")
        buf.write("\u00e7\t\3\2\2\u00e6\u00e5\3\2\2\2\u00e6\u00e7\3\2\2\2")
        buf.write("\u00e7\u00e9\3\2\2\2\u00e8\u00ea\5\20\t\2\u00e9\u00e8")
        buf.write("\3\2\2\2\u00e9\u00ea\3\2\2\2\u00ea\u00ee\3\2\2\2\u00eb")
        buf.write("\u00ed\5\16\b\2\u00ec\u00eb\3\2\2\2\u00ed\u00f0\3\2\2")
        buf.write("\2\u00ee\u00ec\3\2\2\2\u00ee\u00ef\3\2\2\2\u00ef\u00f3")
        buf.write("\3\2\2\2\u00f0\u00ee\3\2\2\2\u00f1\u00f3\7/\2\2\u00f2")
        buf.write("\64\3\2\2\2\u00f2G\3\2\2\2\u00f2f\3\2\2\2\u00f2o\3\2\2")
        buf.write("\2\u00f2v\3\2\2\2\u00f2}\3\2\2\2\u00f2\u0084\3\2\2\2\u00f2")
        buf.write("\u008b\3\2\2\2\u00f2\u009b\3\2\2\2\u00f2\u00ad\3\2\2\2")
        buf.write("\u00f2\u00bf\3\2\2\2\u00f2\u00d1\3\2\2\2\u00f2\u00e1\3")
        buf.write("\2\2\2\u00f2\u00f1\3\2\2\2\u00f3\5\3\2\2\2\u00f4\u00f5")
        buf.write("\7&\2\2\u00f5\7\3\2\2\2\u00f6\u00f7\7\'\2\2\u00f7\t\3")
        buf.write("\2\2\2\u00f8\u00f9\7(\2\2\u00f9\13\3\2\2\2\u00fa\u00fb")
        buf.write("\7)\2\2\u00fb\r\3\2\2\2\u00fc\u00fd\7*\2\2\u00fd\u00fe")
        buf.write("\5\20\t\2\u00fe\17\3\2\2\2\u00ff\u0100\5&\24\2\u0100\u0101")
        buf.write("\7\31\2\2\u0101\u0102\7+\2\2\u0102\21\3\2\2\2\u0103\u0104")
        buf.write("\7\16\2\2\u0104\23\3\2\2\2\u0105\u0106\5&\24\2\u0106\25")
        buf.write("\3\2\2\2\u0107\u010a\5*\26\2\u0108\u010a\5&\24\2\u0109")
        buf.write("\u0107\3\2\2\2\u0109\u0108\3\2\2\2\u010a\27\3\2\2\2\u010b")
        buf.write("\u0110\5$\23\2\u010c\u0110\5*\26\2\u010d\u0110\7+\2\2")
        buf.write("\u010e\u0110\7,\2\2\u010f\u010b\3\2\2\2\u010f\u010c\3")
        buf.write("\2\2\2\u010f\u010d\3\2\2\2\u010f\u010e\3\2\2\2\u0110\31")
        buf.write("\3\2\2\2\u0111\u0112\5$\23\2\u0112\33\3\2\2\2\u0113\u0116")
        buf.write("\5$\23\2\u0114\u0116\5&\24\2\u0115\u0113\3\2\2\2\u0115")
        buf.write("\u0114\3\2\2\2\u0116\35\3\2\2\2\u0117\u0118\5&\24\2\u0118")
        buf.write("\37\3\2\2\2\u0119\u011a\5&\24\2\u011a!\3\2\2\2\u011b\u011c")
        buf.write("\5&\24\2\u011c#\3\2\2\2\u011d\u0122\5&\24\2\u011e\u011f")
        buf.write("\7\5\2\2\u011f\u0121\5&\24\2\u0120\u011e\3\2\2\2\u0121")
        buf.write("\u0124\3\2\2\2\u0122\u0120\3\2\2\2\u0122\u0123\3\2\2\2")
        buf.write("\u0123%\3\2\2\2\u0124\u0122\3\2\2\2\u0125\u0126\5(\25")
        buf.write("\2\u0126\'\3\2\2\2\u0127\u012a\7-\2\2\u0128\u012a\5*\26")
        buf.write("\2\u0129\u0127\3\2\2\2\u0129\u0128\3\2\2\2\u012a)\3\2")
        buf.write("\2\2\u012b\u012c\7.\2\2\u012c+\3\2\2\2*\619<ALNWZ_djs")
        buf.write("z\u0081\u0088\u0090\u0093\u0098\u009e\u00a1\u00a6\u00ab")
        buf.write("\u00b4\u00b7\u00bc\u00c6\u00c9\u00ce\u00d6\u00d9\u00de")
        buf.write("\u00e6\u00e9\u00ee\u00f2\u0109\u010f\u0115\u0122\u0129")
        return buf.getvalue()


class DSLSQLParser ( Parser ):

    grammarFileName = "DSLSQL.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'load'", "'LOAD'", "'.'", "'options'",
                     "'where'", "'as'", "'save'", "'SAVE'", "'partitionBy'",
                     "'select'", "'SELECT'", "';'", "'insert'", "'INSERT'",
                     "'create'", "'CREATE'", "'drop'", "'DROP'", "'refresh'",
                     "'REFRESH'", "'set'", "'SET'", "'='", "'connect'",
                     "'CONNECT'", "'train'", "'TRAIN'", "'run'", "'RUN'",
                     "'register'", "'REGISTER'", "'unRegister'", "'UNREGISTER'",
                     "'include'", "'INCLUDE'", "'overwrite'", "'append'",
                     "'errorIfExists'", "'ignore'", "'and'", "<INVALID>",
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                     "'/**/'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>",
                      "<INVALID>", "STRING", "BLOCK_STRING", "IDENTIFIER",
                      "BACKQUOTED_IDENTIFIER", "SIMPLE_COMMENT", "BRACKETED_EMPTY_COMMENT",
                      "BRACKETED_COMMENT", "WS", "UNRECOGNIZED" ]

    RULE_statement = 0
    RULE_sql = 1
    RULE_overwrite = 2
    RULE_append = 3
    RULE_errorIfExists = 4
    RULE_ignore = 5
    RULE_booleanExpression = 6
    RULE_expression = 7
    RULE_ender = 8
    RULE_format_type = 9
    RULE_path = 10
    RULE_setValue = 11
    RULE_setKey = 12
    RULE_db = 13
    RULE_tableName = 14
    RULE_functionName = 15
    RULE_col = 16
    RULE_qualifiedName = 17
    RULE_identifier = 18
    RULE_strictIdentifier = 19
    RULE_quotedIdentifier = 20

    ruleNames =  [ "statement", "sql", "overwrite", "append", "errorIfExists",
                   "ignore", "booleanExpression", "expression", "ender",
                   "format_type", "path", "setValue", "setKey", "db", "tableName",
                   "functionName", "col", "qualifiedName", "identifier",
                   "strictIdentifier", "quotedIdentifier" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    T__21=22
    T__22=23
    T__23=24
    T__24=25
    T__25=26
    T__26=27
    T__27=28
    T__28=29
    T__29=30
    T__30=31
    T__31=32
    T__32=33
    T__33=34
    T__34=35
    T__35=36
    T__36=37
    T__37=38
    T__38=39
    T__39=40
    STRING=41
    BLOCK_STRING=42
    IDENTIFIER=43
    BACKQUOTED_IDENTIFIER=44
    SIMPLE_COMMENT=45
    BRACKETED_EMPTY_COMMENT=46
    BRACKETED_COMMENT=47
    WS=48
    UNRECOGNIZED=49

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7.1")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None



    class StatementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def sql(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.SqlContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.SqlContext,i)


        def ender(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.EnderContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.EnderContext,i)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStatement" ):
                listener.enterStatement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStatement" ):
                listener.exitStatement(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStatement" ):
                return visitor.visitStatement(self)
            else:
                return visitor.visitChildren(self)




    def statement(self):

        localctx = DSLSQLParser.StatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_statement)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 47
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__0) | (1 << DSLSQLParser.T__1) | (1 << DSLSQLParser.T__6) | (1 << DSLSQLParser.T__7) | (1 << DSLSQLParser.T__9) | (1 << DSLSQLParser.T__10) | (1 << DSLSQLParser.T__12) | (1 << DSLSQLParser.T__13) | (1 << DSLSQLParser.T__14) | (1 << DSLSQLParser.T__15) | (1 << DSLSQLParser.T__16) | (1 << DSLSQLParser.T__17) | (1 << DSLSQLParser.T__18) | (1 << DSLSQLParser.T__19) | (1 << DSLSQLParser.T__20) | (1 << DSLSQLParser.T__21) | (1 << DSLSQLParser.T__23) | (1 << DSLSQLParser.T__24) | (1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28) | (1 << DSLSQLParser.T__29) | (1 << DSLSQLParser.T__30) | (1 << DSLSQLParser.T__31) | (1 << DSLSQLParser.T__32) | (1 << DSLSQLParser.T__33) | (1 << DSLSQLParser.T__34) | (1 << DSLSQLParser.SIMPLE_COMMENT))) != 0):
                self.state = 42
                self.sql()
                self.state = 43
                self.ender()
                self.state = 49
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class SqlContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def format_type(self):
            return self.getTypedRuleContext(DSLSQLParser.Format_typeContext,0)


        def path(self):
            return self.getTypedRuleContext(DSLSQLParser.PathContext,0)


        def tableName(self):
            return self.getTypedRuleContext(DSLSQLParser.TableNameContext,0)


        def expression(self):
            return self.getTypedRuleContext(DSLSQLParser.ExpressionContext,0)


        def booleanExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.BooleanExpressionContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.BooleanExpressionContext,i)


        def overwrite(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.OverwriteContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.OverwriteContext,i)


        def append(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.AppendContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.AppendContext,i)


        def errorIfExists(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.ErrorIfExistsContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.ErrorIfExistsContext,i)


        def ignore(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.IgnoreContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.IgnoreContext,i)


        def col(self):
            return self.getTypedRuleContext(DSLSQLParser.ColContext,0)


        def setKey(self):
            return self.getTypedRuleContext(DSLSQLParser.SetKeyContext,0)


        def setValue(self):
            return self.getTypedRuleContext(DSLSQLParser.SetValueContext,0)


        def db(self):
            return self.getTypedRuleContext(DSLSQLParser.DbContext,0)


        def functionName(self):
            return self.getTypedRuleContext(DSLSQLParser.FunctionNameContext,0)


        def SIMPLE_COMMENT(self):
            return self.getToken(DSLSQLParser.SIMPLE_COMMENT, 0)

        def getRuleIndex(self):
            return DSLSQLParser.RULE_sql

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSql" ):
                listener.enterSql(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSql" ):
                listener.exitSql(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSql" ):
                return visitor.visitSql(self)
            else:
                return visitor.visitChildren(self)




    def sql(self):

        localctx = DSLSQLParser.SqlContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_sql)
        self._la = 0 # Token type
        try:
            self.state = 240
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [DSLSQLParser.T__0, DSLSQLParser.T__1]:
                self.enterOuterAlt(localctx, 1)
                self.state = 50
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__0 or _la==DSLSQLParser.T__1):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 51
                self.format_type()
                self.state = 52
                self.match(DSLSQLParser.T__2)
                self.state = 53
                self.path()
                self.state = 55
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 54
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 58
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 57
                    self.expression()


                self.state = 63
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 60
                    self.booleanExpression()
                    self.state = 65
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 66
                self.match(DSLSQLParser.T__5)
                self.state = 67
                self.tableName()
                pass
            elif token in [DSLSQLParser.T__6, DSLSQLParser.T__7]:
                self.enterOuterAlt(localctx, 2)
                self.state = 69
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__6 or _la==DSLSQLParser.T__7):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 76
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__35) | (1 << DSLSQLParser.T__36) | (1 << DSLSQLParser.T__37) | (1 << DSLSQLParser.T__38))) != 0):
                    self.state = 74
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [DSLSQLParser.T__35]:
                        self.state = 70
                        self.overwrite()
                        pass
                    elif token in [DSLSQLParser.T__36]:
                        self.state = 71
                        self.append()
                        pass
                    elif token in [DSLSQLParser.T__37]:
                        self.state = 72
                        self.errorIfExists()
                        pass
                    elif token in [DSLSQLParser.T__38]:
                        self.state = 73
                        self.ignore()
                        pass
                    else:
                        raise NoViableAltException(self)

                    self.state = 78
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 79
                self.tableName()
                self.state = 80
                self.match(DSLSQLParser.T__5)
                self.state = 81
                self.format_type()
                self.state = 82
                self.match(DSLSQLParser.T__2)
                self.state = 83
                self.path()
                self.state = 85
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 84
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 88
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 87
                    self.expression()


                self.state = 93
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 90
                    self.booleanExpression()
                    self.state = 95
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 98
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__8:
                    self.state = 96
                    self.match(DSLSQLParser.T__8)
                    self.state = 97
                    self.col()


                pass
            elif token in [DSLSQLParser.T__9, DSLSQLParser.T__10]:
                self.enterOuterAlt(localctx, 3)
                self.state = 100
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__9 or _la==DSLSQLParser.T__10):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 104
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,10,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 101
                        _la = self._input.LA(1)
                        if _la <= 0 or _la==DSLSQLParser.T__11:
                            self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()
                    self.state = 106
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,10,self._ctx)

                self.state = 107
                self.match(DSLSQLParser.T__5)
                self.state = 108
                self.tableName()
                pass
            elif token in [DSLSQLParser.T__12, DSLSQLParser.T__13]:
                self.enterOuterAlt(localctx, 4)
                self.state = 109
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__12 or _la==DSLSQLParser.T__13):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 113
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__0) | (1 << DSLSQLParser.T__1) | (1 << DSLSQLParser.T__2) | (1 << DSLSQLParser.T__3) | (1 << DSLSQLParser.T__4) | (1 << DSLSQLParser.T__5) | (1 << DSLSQLParser.T__6) | (1 << DSLSQLParser.T__7) | (1 << DSLSQLParser.T__8) | (1 << DSLSQLParser.T__9) | (1 << DSLSQLParser.T__10) | (1 << DSLSQLParser.T__12) | (1 << DSLSQLParser.T__13) | (1 << DSLSQLParser.T__14) | (1 << DSLSQLParser.T__15) | (1 << DSLSQLParser.T__16) | (1 << DSLSQLParser.T__17) | (1 << DSLSQLParser.T__18) | (1 << DSLSQLParser.T__19) | (1 << DSLSQLParser.T__20) | (1 << DSLSQLParser.T__21) | (1 << DSLSQLParser.T__22) | (1 << DSLSQLParser.T__23) | (1 << DSLSQLParser.T__24) | (1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28) | (1 << DSLSQLParser.T__29) | (1 << DSLSQLParser.T__30) | (1 << DSLSQLParser.T__31) | (1 << DSLSQLParser.T__32) | (1 << DSLSQLParser.T__33) | (1 << DSLSQLParser.T__34) | (1 << DSLSQLParser.T__35) | (1 << DSLSQLParser.T__36) | (1 << DSLSQLParser.T__37) | (1 << DSLSQLParser.T__38) | (1 << DSLSQLParser.T__39) | (1 << DSLSQLParser.STRING) | (1 << DSLSQLParser.BLOCK_STRING) | (1 << DSLSQLParser.IDENTIFIER) | (1 << DSLSQLParser.BACKQUOTED_IDENTIFIER) | (1 << DSLSQLParser.SIMPLE_COMMENT) | (1 << DSLSQLParser.BRACKETED_EMPTY_COMMENT) | (1 << DSLSQLParser.BRACKETED_COMMENT) | (1 << DSLSQLParser.WS) | (1 << DSLSQLParser.UNRECOGNIZED))) != 0):
                    self.state = 110
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==DSLSQLParser.T__11:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 115
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__14, DSLSQLParser.T__15]:
                self.enterOuterAlt(localctx, 5)
                self.state = 116
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__14 or _la==DSLSQLParser.T__15):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 120
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__0) | (1 << DSLSQLParser.T__1) | (1 << DSLSQLParser.T__2) | (1 << DSLSQLParser.T__3) | (1 << DSLSQLParser.T__4) | (1 << DSLSQLParser.T__5) | (1 << DSLSQLParser.T__6) | (1 << DSLSQLParser.T__7) | (1 << DSLSQLParser.T__8) | (1 << DSLSQLParser.T__9) | (1 << DSLSQLParser.T__10) | (1 << DSLSQLParser.T__12) | (1 << DSLSQLParser.T__13) | (1 << DSLSQLParser.T__14) | (1 << DSLSQLParser.T__15) | (1 << DSLSQLParser.T__16) | (1 << DSLSQLParser.T__17) | (1 << DSLSQLParser.T__18) | (1 << DSLSQLParser.T__19) | (1 << DSLSQLParser.T__20) | (1 << DSLSQLParser.T__21) | (1 << DSLSQLParser.T__22) | (1 << DSLSQLParser.T__23) | (1 << DSLSQLParser.T__24) | (1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28) | (1 << DSLSQLParser.T__29) | (1 << DSLSQLParser.T__30) | (1 << DSLSQLParser.T__31) | (1 << DSLSQLParser.T__32) | (1 << DSLSQLParser.T__33) | (1 << DSLSQLParser.T__34) | (1 << DSLSQLParser.T__35) | (1 << DSLSQLParser.T__36) | (1 << DSLSQLParser.T__37) | (1 << DSLSQLParser.T__38) | (1 << DSLSQLParser.T__39) | (1 << DSLSQLParser.STRING) | (1 << DSLSQLParser.BLOCK_STRING) | (1 << DSLSQLParser.IDENTIFIER) | (1 << DSLSQLParser.BACKQUOTED_IDENTIFIER) | (1 << DSLSQLParser.SIMPLE_COMMENT) | (1 << DSLSQLParser.BRACKETED_EMPTY_COMMENT) | (1 << DSLSQLParser.BRACKETED_COMMENT) | (1 << DSLSQLParser.WS) | (1 << DSLSQLParser.UNRECOGNIZED))) != 0):
                    self.state = 117
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==DSLSQLParser.T__11:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 122
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__16, DSLSQLParser.T__17]:
                self.enterOuterAlt(localctx, 6)
                self.state = 123
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__16 or _la==DSLSQLParser.T__17):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 127
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__0) | (1 << DSLSQLParser.T__1) | (1 << DSLSQLParser.T__2) | (1 << DSLSQLParser.T__3) | (1 << DSLSQLParser.T__4) | (1 << DSLSQLParser.T__5) | (1 << DSLSQLParser.T__6) | (1 << DSLSQLParser.T__7) | (1 << DSLSQLParser.T__8) | (1 << DSLSQLParser.T__9) | (1 << DSLSQLParser.T__10) | (1 << DSLSQLParser.T__12) | (1 << DSLSQLParser.T__13) | (1 << DSLSQLParser.T__14) | (1 << DSLSQLParser.T__15) | (1 << DSLSQLParser.T__16) | (1 << DSLSQLParser.T__17) | (1 << DSLSQLParser.T__18) | (1 << DSLSQLParser.T__19) | (1 << DSLSQLParser.T__20) | (1 << DSLSQLParser.T__21) | (1 << DSLSQLParser.T__22) | (1 << DSLSQLParser.T__23) | (1 << DSLSQLParser.T__24) | (1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28) | (1 << DSLSQLParser.T__29) | (1 << DSLSQLParser.T__30) | (1 << DSLSQLParser.T__31) | (1 << DSLSQLParser.T__32) | (1 << DSLSQLParser.T__33) | (1 << DSLSQLParser.T__34) | (1 << DSLSQLParser.T__35) | (1 << DSLSQLParser.T__36) | (1 << DSLSQLParser.T__37) | (1 << DSLSQLParser.T__38) | (1 << DSLSQLParser.T__39) | (1 << DSLSQLParser.STRING) | (1 << DSLSQLParser.BLOCK_STRING) | (1 << DSLSQLParser.IDENTIFIER) | (1 << DSLSQLParser.BACKQUOTED_IDENTIFIER) | (1 << DSLSQLParser.SIMPLE_COMMENT) | (1 << DSLSQLParser.BRACKETED_EMPTY_COMMENT) | (1 << DSLSQLParser.BRACKETED_COMMENT) | (1 << DSLSQLParser.WS) | (1 << DSLSQLParser.UNRECOGNIZED))) != 0):
                    self.state = 124
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==DSLSQLParser.T__11:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 129
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__18, DSLSQLParser.T__19]:
                self.enterOuterAlt(localctx, 7)
                self.state = 130
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__18 or _la==DSLSQLParser.T__19):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 134
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__0) | (1 << DSLSQLParser.T__1) | (1 << DSLSQLParser.T__2) | (1 << DSLSQLParser.T__3) | (1 << DSLSQLParser.T__4) | (1 << DSLSQLParser.T__5) | (1 << DSLSQLParser.T__6) | (1 << DSLSQLParser.T__7) | (1 << DSLSQLParser.T__8) | (1 << DSLSQLParser.T__9) | (1 << DSLSQLParser.T__10) | (1 << DSLSQLParser.T__12) | (1 << DSLSQLParser.T__13) | (1 << DSLSQLParser.T__14) | (1 << DSLSQLParser.T__15) | (1 << DSLSQLParser.T__16) | (1 << DSLSQLParser.T__17) | (1 << DSLSQLParser.T__18) | (1 << DSLSQLParser.T__19) | (1 << DSLSQLParser.T__20) | (1 << DSLSQLParser.T__21) | (1 << DSLSQLParser.T__22) | (1 << DSLSQLParser.T__23) | (1 << DSLSQLParser.T__24) | (1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28) | (1 << DSLSQLParser.T__29) | (1 << DSLSQLParser.T__30) | (1 << DSLSQLParser.T__31) | (1 << DSLSQLParser.T__32) | (1 << DSLSQLParser.T__33) | (1 << DSLSQLParser.T__34) | (1 << DSLSQLParser.T__35) | (1 << DSLSQLParser.T__36) | (1 << DSLSQLParser.T__37) | (1 << DSLSQLParser.T__38) | (1 << DSLSQLParser.T__39) | (1 << DSLSQLParser.STRING) | (1 << DSLSQLParser.BLOCK_STRING) | (1 << DSLSQLParser.IDENTIFIER) | (1 << DSLSQLParser.BACKQUOTED_IDENTIFIER) | (1 << DSLSQLParser.SIMPLE_COMMENT) | (1 << DSLSQLParser.BRACKETED_EMPTY_COMMENT) | (1 << DSLSQLParser.BRACKETED_COMMENT) | (1 << DSLSQLParser.WS) | (1 << DSLSQLParser.UNRECOGNIZED))) != 0):
                    self.state = 131
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==DSLSQLParser.T__11:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 136
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__20, DSLSQLParser.T__21]:
                self.enterOuterAlt(localctx, 8)
                self.state = 137
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__20 or _la==DSLSQLParser.T__21):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 138
                self.setKey()
                self.state = 139
                self.match(DSLSQLParser.T__22)
                self.state = 140
                self.setValue()
                self.state = 142
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 141
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 145
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 144
                    self.expression()


                self.state = 150
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 147
                    self.booleanExpression()
                    self.state = 152
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__23, DSLSQLParser.T__24]:
                self.enterOuterAlt(localctx, 9)
                self.state = 153
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__23 or _la==DSLSQLParser.T__24):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 154
                self.format_type()
                self.state = 156
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 155
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 159
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 158
                    self.expression()


                self.state = 164
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 161
                    self.booleanExpression()
                    self.state = 166
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 169
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__5:
                    self.state = 167
                    self.match(DSLSQLParser.T__5)
                    self.state = 168
                    self.db()


                pass
            elif token in [DSLSQLParser.T__25, DSLSQLParser.T__26, DSLSQLParser.T__27, DSLSQLParser.T__28]:
                self.enterOuterAlt(localctx, 10)
                self.state = 171
                _la = self._input.LA(1)
                if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << DSLSQLParser.T__25) | (1 << DSLSQLParser.T__26) | (1 << DSLSQLParser.T__27) | (1 << DSLSQLParser.T__28))) != 0)):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 172
                self.tableName()
                self.state = 173
                self.match(DSLSQLParser.T__5)
                self.state = 174
                self.format_type()
                self.state = 175
                self.match(DSLSQLParser.T__2)
                self.state = 176
                self.path()
                self.state = 178
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 177
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 181
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 180
                    self.expression()


                self.state = 186
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 183
                    self.booleanExpression()
                    self.state = 188
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__29, DSLSQLParser.T__30]:
                self.enterOuterAlt(localctx, 11)
                self.state = 189
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__29 or _la==DSLSQLParser.T__30):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 190
                self.format_type()
                self.state = 191
                self.match(DSLSQLParser.T__2)
                self.state = 192
                self.path()
                self.state = 193
                self.match(DSLSQLParser.T__5)
                self.state = 194
                self.functionName()
                self.state = 196
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 195
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 199
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 198
                    self.expression()


                self.state = 204
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 201
                    self.booleanExpression()
                    self.state = 206
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__31, DSLSQLParser.T__32]:
                self.enterOuterAlt(localctx, 12)
                self.state = 207
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__31 or _la==DSLSQLParser.T__32):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 208
                self.format_type()
                self.state = 209
                self.match(DSLSQLParser.T__2)
                self.state = 210
                self.path()
                self.state = 212
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 211
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 215
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 214
                    self.expression()


                self.state = 220
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 217
                    self.booleanExpression()
                    self.state = 222
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.T__33, DSLSQLParser.T__34]:
                self.enterOuterAlt(localctx, 13)
                self.state = 223
                _la = self._input.LA(1)
                if not(_la==DSLSQLParser.T__33 or _la==DSLSQLParser.T__34):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 224
                self.format_type()
                self.state = 225
                self.match(DSLSQLParser.T__2)
                self.state = 226
                self.path()
                self.state = 228
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4:
                    self.state = 227
                    _la = self._input.LA(1)
                    if not(_la==DSLSQLParser.T__3 or _la==DSLSQLParser.T__4):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()


                self.state = 231
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==DSLSQLParser.IDENTIFIER or _la==DSLSQLParser.BACKQUOTED_IDENTIFIER:
                    self.state = 230
                    self.expression()


                self.state = 236
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==DSLSQLParser.T__39:
                    self.state = 233
                    self.booleanExpression()
                    self.state = 238
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [DSLSQLParser.SIMPLE_COMMENT]:
                self.enterOuterAlt(localctx, 14)
                self.state = 239
                self.match(DSLSQLParser.SIMPLE_COMMENT)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class OverwriteContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return DSLSQLParser.RULE_overwrite

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOverwrite" ):
                listener.enterOverwrite(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOverwrite" ):
                listener.exitOverwrite(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOverwrite" ):
                return visitor.visitOverwrite(self)
            else:
                return visitor.visitChildren(self)




    def overwrite(self):

        localctx = DSLSQLParser.OverwriteContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_overwrite)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 242
            self.match(DSLSQLParser.T__35)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class AppendContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return DSLSQLParser.RULE_append

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAppend" ):
                listener.enterAppend(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAppend" ):
                listener.exitAppend(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAppend" ):
                return visitor.visitAppend(self)
            else:
                return visitor.visitChildren(self)




    def append(self):

        localctx = DSLSQLParser.AppendContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_append)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 244
            self.match(DSLSQLParser.T__36)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ErrorIfExistsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return DSLSQLParser.RULE_errorIfExists

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterErrorIfExists" ):
                listener.enterErrorIfExists(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitErrorIfExists" ):
                listener.exitErrorIfExists(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitErrorIfExists" ):
                return visitor.visitErrorIfExists(self)
            else:
                return visitor.visitChildren(self)




    def errorIfExists(self):

        localctx = DSLSQLParser.ErrorIfExistsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_errorIfExists)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 246
            self.match(DSLSQLParser.T__37)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class IgnoreContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return DSLSQLParser.RULE_ignore

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIgnore" ):
                listener.enterIgnore(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIgnore" ):
                listener.exitIgnore(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitIgnore" ):
                return visitor.visitIgnore(self)
            else:
                return visitor.visitChildren(self)




    def ignore(self):

        localctx = DSLSQLParser.IgnoreContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_ignore)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 248
            self.match(DSLSQLParser.T__38)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class BooleanExpressionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(DSLSQLParser.ExpressionContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_booleanExpression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBooleanExpression" ):
                listener.enterBooleanExpression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBooleanExpression" ):
                listener.exitBooleanExpression(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBooleanExpression" ):
                return visitor.visitBooleanExpression(self)
            else:
                return visitor.visitChildren(self)




    def booleanExpression(self):

        localctx = DSLSQLParser.BooleanExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_booleanExpression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 250
            self.match(DSLSQLParser.T__39)
            self.state = 251
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ExpressionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def STRING(self):
            return self.getToken(DSLSQLParser.STRING, 0)

        def getRuleIndex(self):
            return DSLSQLParser.RULE_expression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExpression" ):
                listener.enterExpression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExpression" ):
                listener.exitExpression(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExpression" ):
                return visitor.visitExpression(self)
            else:
                return visitor.visitChildren(self)




    def expression(self):

        localctx = DSLSQLParser.ExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_expression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 253
            self.identifier()
            self.state = 254
            self.match(DSLSQLParser.T__22)
            self.state = 255
            self.match(DSLSQLParser.STRING)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class EnderContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return DSLSQLParser.RULE_ender

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEnder" ):
                listener.enterEnder(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEnder" ):
                listener.exitEnder(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEnder" ):
                return visitor.visitEnder(self)
            else:
                return visitor.visitChildren(self)




    def ender(self):

        localctx = DSLSQLParser.EnderContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_ender)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 257
            self.match(DSLSQLParser.T__11)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Format_typeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_format_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFormat_type" ):
                listener.enterFormat_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFormat_type" ):
                listener.exitFormat_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFormat_type" ):
                return visitor.visitFormat_type(self)
            else:
                return visitor.visitChildren(self)




    def format_type(self):

        localctx = DSLSQLParser.Format_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_format_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 259
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class PathContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def quotedIdentifier(self):
            return self.getTypedRuleContext(DSLSQLParser.QuotedIdentifierContext,0)


        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_path

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPath" ):
                listener.enterPath(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPath" ):
                listener.exitPath(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPath" ):
                return visitor.visitPath(self)
            else:
                return visitor.visitChildren(self)




    def path(self):

        localctx = DSLSQLParser.PathContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_path)
        try:
            self.state = 263
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,35,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 261
                self.quotedIdentifier()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 262
                self.identifier()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class SetValueContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def qualifiedName(self):
            return self.getTypedRuleContext(DSLSQLParser.QualifiedNameContext,0)


        def quotedIdentifier(self):
            return self.getTypedRuleContext(DSLSQLParser.QuotedIdentifierContext,0)


        def STRING(self):
            return self.getToken(DSLSQLParser.STRING, 0)

        def BLOCK_STRING(self):
            return self.getToken(DSLSQLParser.BLOCK_STRING, 0)

        def getRuleIndex(self):
            return DSLSQLParser.RULE_setValue

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSetValue" ):
                listener.enterSetValue(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSetValue" ):
                listener.exitSetValue(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSetValue" ):
                return visitor.visitSetValue(self)
            else:
                return visitor.visitChildren(self)




    def setValue(self):

        localctx = DSLSQLParser.SetValueContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_setValue)
        try:
            self.state = 269
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,36,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 265
                self.qualifiedName()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 266
                self.quotedIdentifier()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 267
                self.match(DSLSQLParser.STRING)
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 268
                self.match(DSLSQLParser.BLOCK_STRING)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class SetKeyContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def qualifiedName(self):
            return self.getTypedRuleContext(DSLSQLParser.QualifiedNameContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_setKey

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSetKey" ):
                listener.enterSetKey(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSetKey" ):
                listener.exitSetKey(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSetKey" ):
                return visitor.visitSetKey(self)
            else:
                return visitor.visitChildren(self)




    def setKey(self):

        localctx = DSLSQLParser.SetKeyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_setKey)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 271
            self.qualifiedName()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DbContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def qualifiedName(self):
            return self.getTypedRuleContext(DSLSQLParser.QualifiedNameContext,0)


        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_db

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDb" ):
                listener.enterDb(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDb" ):
                listener.exitDb(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDb" ):
                return visitor.visitDb(self)
            else:
                return visitor.visitChildren(self)




    def db(self):

        localctx = DSLSQLParser.DbContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_db)
        try:
            self.state = 275
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,37,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 273
                self.qualifiedName()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 274
                self.identifier()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class TableNameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_tableName

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTableName" ):
                listener.enterTableName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTableName" ):
                listener.exitTableName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTableName" ):
                return visitor.visitTableName(self)
            else:
                return visitor.visitChildren(self)




    def tableName(self):

        localctx = DSLSQLParser.TableNameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_tableName)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 277
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class FunctionNameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_functionName

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunctionName" ):
                listener.enterFunctionName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunctionName" ):
                listener.exitFunctionName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunctionName" ):
                return visitor.visitFunctionName(self)
            else:
                return visitor.visitChildren(self)




    def functionName(self):

        localctx = DSLSQLParser.FunctionNameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_functionName)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 279
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ColContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_col

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCol" ):
                listener.enterCol(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCol" ):
                listener.exitCol(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCol" ):
                return visitor.visitCol(self)
            else:
                return visitor.visitChildren(self)




    def col(self):

        localctx = DSLSQLParser.ColContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_col)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 281
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class QualifiedNameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(DSLSQLParser.IdentifierContext)
            else:
                return self.getTypedRuleContext(DSLSQLParser.IdentifierContext,i)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_qualifiedName

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterQualifiedName" ):
                listener.enterQualifiedName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitQualifiedName" ):
                listener.exitQualifiedName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitQualifiedName" ):
                return visitor.visitQualifiedName(self)
            else:
                return visitor.visitChildren(self)




    def qualifiedName(self):

        localctx = DSLSQLParser.QualifiedNameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_qualifiedName)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 283
            self.identifier()
            self.state = 288
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==DSLSQLParser.T__2:
                self.state = 284
                self.match(DSLSQLParser.T__2)
                self.state = 285
                self.identifier()
                self.state = 290
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class IdentifierContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def strictIdentifier(self):
            return self.getTypedRuleContext(DSLSQLParser.StrictIdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_identifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIdentifier" ):
                listener.enterIdentifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIdentifier" ):
                listener.exitIdentifier(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitIdentifier" ):
                return visitor.visitIdentifier(self)
            else:
                return visitor.visitChildren(self)




    def identifier(self):

        localctx = DSLSQLParser.IdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_identifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 291
            self.strictIdentifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class StrictIdentifierContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IDENTIFIER(self):
            return self.getToken(DSLSQLParser.IDENTIFIER, 0)

        def quotedIdentifier(self):
            return self.getTypedRuleContext(DSLSQLParser.QuotedIdentifierContext,0)


        def getRuleIndex(self):
            return DSLSQLParser.RULE_strictIdentifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStrictIdentifier" ):
                listener.enterStrictIdentifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStrictIdentifier" ):
                listener.exitStrictIdentifier(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStrictIdentifier" ):
                return visitor.visitStrictIdentifier(self)
            else:
                return visitor.visitChildren(self)




    def strictIdentifier(self):

        localctx = DSLSQLParser.StrictIdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_strictIdentifier)
        try:
            self.state = 295
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [DSLSQLParser.IDENTIFIER]:
                self.enterOuterAlt(localctx, 1)
                self.state = 293
                self.match(DSLSQLParser.IDENTIFIER)
                pass
            elif token in [DSLSQLParser.BACKQUOTED_IDENTIFIER]:
                self.enterOuterAlt(localctx, 2)
                self.state = 294
                self.quotedIdentifier()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class QuotedIdentifierContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BACKQUOTED_IDENTIFIER(self):
            return self.getToken(DSLSQLParser.BACKQUOTED_IDENTIFIER, 0)

        def getRuleIndex(self):
            return DSLSQLParser.RULE_quotedIdentifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterQuotedIdentifier" ):
                listener.enterQuotedIdentifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitQuotedIdentifier" ):
                listener.exitQuotedIdentifier(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitQuotedIdentifier" ):
                return visitor.visitQuotedIdentifier(self)
            else:
                return visitor.visitChildren(self)




    def quotedIdentifier(self):

        localctx = DSLSQLParser.QuotedIdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_quotedIdentifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 297
            self.match(DSLSQLParser.BACKQUOTED_IDENTIFIER)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx







import os
import platform
import subprocess
import sys

import psutil


def get_windows_parent_process_name():
    """
    获取当前进程的父进程名（仅在Windows系统下有意义）。

    适用场景：
    - 判断命令是否由PowerShell或cmd.exe启动，以便调整命令格式。
    - 在Windows平台上进行父进程分析。

    参数：
    - 无

    返回：
    - str|None: 父进程名（小写字符串，如"powershell.exe"或"cmd.exe"），如果无法获取则为None。

    异常：
    - 捕获所有异常，返回None。
    """
    try:
        current_process = psutil.Process()
        while True:
            parent = current_process.parent()
            if parent is None:
                break
            parent_name = parent.name().lower()
            if parent_name in ["powershell.exe", "cmd.exe"]:
                return parent_name
            current_process = parent
        return None
    except Exception:
        return None


def run_cmd_subprocess(command, verbose=False, cwd=None, encoding=sys.stdout.encoding):
    if verbose:
        print("Using run_cmd_subprocess:", command)

    try:
        shell = os.environ.get("SHELL", "/bin/sh")
        parent_process = None

        # Determine the appropriate shell
        if platform.system() == "Windows":
            parent_process = get_windows_parent_process_name()
            if parent_process == "powershell.exe":
                command = f"powershell -Command {command}"

        if verbose:
            print("Running command:", command)
            print("SHELL:", shell)
            if platform.system() == "Windows":
                print("Parent process:", parent_process)

        process = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,
            shell=True,
            encoding=encoding,
            errors="replace",
            bufsize=0,  # Set bufsize to 0 for unbuffered output
            universal_newlines=True,
            cwd=cwd,
        )

        output = []
        while True:
            chunk = process.stdout.read(1)
            if not chunk:
                break
            # print(chunk, end="", flush=True)  # Print the chunk in real-time
            output.append(chunk)  # Store the chunk for later use

        process.wait()
        return process.returncode, "".join(output)
    except Exception as e:
        return 1, str(e)


import json
import os
import statistics
import time
import traceback
from importlib import resources
from typing import Optional, Dict, Any, Generator, List

import pathspec
from tokenizers import Tokenizer

from autocoder_nano.core import AutoLLM
from autocoder_nano.core import prompt
from autocoder_nano.actypes import SourceCode, AutoCoderArgs, FilterDoc, VariableHolder
from autocoder_nano.rag.doc_filter import DocFilter
from autocoder_nano.rag.doc_limiter import TokenLimiter
from autocoder_nano.rag.doc_retriever import LocalDocumentRetriever
from autocoder_nano.rag.token_counter import TokenCounter, RemoteTokenCounter
from autocoder_nano.utils.printer_utils import Printer


printer = Printer()


def get_event_file_path(request_id: str, base_path: str = "events") -> str:
    return f"{base_path}/{request_id}.jsonl"


class LongContextRAG:
    def __init__(self, llm: AutoLLM, args: AutoCoderArgs, path: str, tokenizer_path: Optional[str] = None,) -> None:
        self.llm = llm
        self.args = args
        self.path = path
        if not os.path.exists(self.path):
            raise ValueError("请提供本地文件系统中文档的路径.")

        # 基础配置
        self.relevant_score = self.args.rag_doc_filter_relevance or 5
        self.token_limit = self.args.rag_context_window_limit or 120000
        self.full_text_ratio = self.args.full_text_ratio
        self.segment_ratio = self.args.segment_ratio
        self.buff_ratio = 1 - self.full_text_ratio - self.segment_ratio
        if self.buff_ratio < 0:
            raise ValueError("full_text_ratio 和 segment_ratio 的总和必须小于或等于 1.0")
        self.full_text_limit = int(self.args.rag_context_window_limit * self.full_text_ratio)
        self.segment_limit = int(self.args.rag_context_window_limit * self.segment_ratio)
        self.buff_limit = int(self.args.rag_context_window_limit * self.buff_ratio)

        # 分词相关
        self.tokenizer = None
        if tokenizer_path:
            self.tokenizer_path = tokenizer_path
        else:
            self.tokenizer_path = resources.files("autocoder_nano").joinpath("data/tokenizer.json").__str__()
        self.args.tokenizer_path = self.tokenizer_path
        VariableHolder.TOKENIZER_PATH = self.tokenizer_path
        VariableHolder.TOKENIZER_MODEL = Tokenizer.from_file(self.tokenizer_path)
        self.tokenizer = TokenCounter(self.tokenizer_path)

        # 设置忽略文件后缀
        self.ignore_spec = self._load_ignore_file()
        self.required_exts = (
            [ext.strip() for ext in self.args.required_exts.split(",")]
            if self.args.required_exts
            else []
        )

        # 监控模式
        self.monitor_mode = self.args.monitor_mode or False
        self.enable_hybrid_index = self.args.enable_hybrid_index

        retriever_class = self._get_document_retriever_class()
        self.document_retriever = retriever_class(
            self.llm,
            self.args,
            self.path,
            self.ignore_spec,
            self.required_exts,
            False,
            self.monitor_mode,
            # 确保全文区至少能放下一个文件
            single_file_token_limit=self.full_text_limit - 100,
            disable_auto_window=self.args.disable_auto_window,
            enable_hybrid_index=self.args.enable_hybrid_index
        )

        # 文本过滤初始化
        self.doc_filter = DocFilter(
            self.llm, self.args, on_ray=False, path=self.path
        )

        doc_num = 0
        token_num = 0
        token_counts = []
        for doc in self._retrieve_documents():
            doc_num += 1
            doc_tokens = doc.tokens
            token_num += doc_tokens
            token_counts.append(doc_tokens)
        avg_tokens = statistics.mean(token_counts) if token_counts else 0
        median_tokens = statistics.median(token_counts) if token_counts else 0

        printer.print_key_value(
            items={
                "监控模式": f"{self.monitor_mode}",
                "文档总数": f"{doc_num}",
                "总 Token 数": f"{token_num}",
                "分词器路径": f"{self.tokenizer_path}",
                "相关性分数": f"{self.relevant_score}",
                "Token 限制": f"{self.token_limit}",
                "全文限制": f"{self.monitor_mode}",
                "段落限制": f"{self.segment_limit}",
                "缓冲限制": f"{self.buff_limit}",
                "最大文档 Token 数": f"{max(token_counts) if token_counts else 0}",
                "最小文档 Token 数": f"{min(token_counts) if token_counts else 0}",
                "平均文档 Token 数": f"{avg_tokens:.2f}",
                "文档 Token 数中位数": f"{median_tokens:.2f}"
            },
            title="Long Context RAG 配置:"
        )

    def count_tokens(self, text: str) -> int:
        if self.tokenizer is None:
            return -1
        return self.tokenizer.count_tokens(text)

    @prompt()
    def extract_relevance_info_from_docs_with_conversation(
            self, conversations: List[Dict[str, str]], documents: List[str]
    ) -> str:
        """
        使用以下文档和对话历史来提取相关信息。

        文档：
        <documents>
        {% for doc in documents %}
        {{ doc }}
        {% endfor %}
        </documents>

        对话历史：
        <conversations>
        {% for msg in conversations %}
        [{{ msg.role }}]:
        {{ msg.content }}

        {% endfor %}
        </conversations>

        请根据提供的文档内容、用户对话历史以及最后一个问题，提取并总结文档中与问题相关的重要信息。
        如果文档中没有相关信息，请回复"该文档中没有与问题相关的信息"。
        提取的信息尽量保持和原文中的一样，并且只输出这些信息。
        """

    @prompt()
    def _answer_question(
            self, query: str, relevant_docs: List[str]
    ) -> Generator[str, None, None]:
        """
        文档：
        <documents>
        {% for doc in relevant_docs %}
        {{ doc }}
        {% endfor %}
        </documents>

        使用以上文档来回答用户的问题。回答要求：

        1. 严格基于文档内容回答
        - 如果文档提供的信息无法回答问题,请明确回复:"抱歉,文档中没有足够的信息来回答这个问题。"
        - 不要添加、推测或扩展文档未提及的信息

        2. 格式如 ![image](./path.png) 的 Markdown 图片处理
        - 根据Markdown 图片前后文本内容推测改图片与问题的相关性，有相关性则在回答中输出该Markdown图片路径
        - 根据相关图片在文档中的位置，自然融入答复内容,保持上下文连贯
        - 完整保留原始图片路径,不省略任何部分

        3. 回答格式要求
        - 使用markdown格式提升可读性

        问题：{{ query }}
        """

    def _retrieve_documents(self, options: Optional[Dict[str, Any]] = None) -> Generator[SourceCode, None, None]:
        return self.document_retriever.retrieve_documents(options=options)

    @staticmethod
    def _get_document_retriever_class():
        """Get the document retriever class based on configuration."""
        # Default to LocalDocumentRetriever if not specified
        return LocalDocumentRetriever

    def _load_ignore_file(self):
        serveignore_path = os.path.join(self.path, ".serveignore")
        gitignore_path = os.path.join(self.path, ".gitignore")

        if os.path.exists(serveignore_path):
            with open(serveignore_path, "r") as ignore_file:
                return pathspec.PathSpec.from_lines("gitwildmatch", ignore_file)
        elif os.path.exists(gitignore_path):
            with open(gitignore_path, "r") as ignore_file:
                return pathspec.PathSpec.from_lines("gitwildmatch", ignore_file)
        return None

    def build_cache(self):
        self.document_retriever.build_cache()

    def search(self, query: str) -> List[SourceCode]:
        target_query = query
        only_contexts = False
        if self.args.enable_rag_search and isinstance(self.args.enable_rag_search, str):
            target_query = self.args.enable_rag_search
        elif self.args.enable_rag_context and isinstance(self.args.enable_rag_context, str):
            target_query = self.args.enable_rag_context
            only_contexts = True
        elif self.args.enable_rag_context:
            only_contexts = True

        printer.print_key_value(
            items={"查询内容": f"{target_query[0:50]}...", "仅返回正文": f"{only_contexts}"},
            title="正在从 RAG 中搜索....."
        )

        # 设置的本地 path 类型的 rag
        if only_contexts:
            return [
                doc.source_code
                for doc in self._filter_docs(
                    [{"role": "user", "content": target_query}]
                )
            ]
        else:
            v, contexts = self.stream_chat_oai(
                conversations=[{"role": "user", "content": target_query}]
            )
            url = ",".join(contexts)
            return [SourceCode(module_name=f"RAG:{url}", source_code="".join(v))]

    def _filter_docs(self, conversations: List[Dict[str, str]]) -> List[FilterDoc]:
        query = conversations[-1]["content"]
        documents = self._retrieve_documents(options={"query": query})
        return self.doc_filter.filter_docs(
            conversations=conversations, documents=[d for d in documents]
        )

    def stream_chat_oai(self, conversations, model: Optional[str] = None):
        try:
            return self._stream_chat_oai(conversations, model=model)
        except Exception as e:
            printer.print_text(f"Error in stream_chat_oai: {str(e)}", style="red")
            traceback.print_exc()
            return ["出现错误，请稍后再试。"], []

    def _stream_chat_oai(self, conversations, model: Optional[str] = None):
        query = conversations[-1]["content"]
        context = []
        only_contexts = False
        try:
            v = json.loads(query)
            if "only_contexts" in v:
                query = v["query"]
                only_contexts = v["only_contexts"]
                conversations[-1]["content"] = query
        except json.JSONDecodeError:
            pass

        start_time = time.time()
        relevant_docs: List[FilterDoc] = self._filter_docs(conversations)
        filter_time = time.time() - start_time

        # 过滤 relevant_docs，仅包含 is_relevant=True 的文档
        highly_relevant_docs = [
            doc for doc in relevant_docs if doc.relevance.is_relevant
        ]

        if highly_relevant_docs:
            relevant_docs = highly_relevant_docs
        printer.print_key_value(
            items={"相关文档数量": f"{len(relevant_docs)}", "过滤耗时": f"{filter_time:.2f} 秒"},
            title="Filter Docs"
        )

        if only_contexts:  # 仅返回相关文档原文
            final_docs = []
            for doc in relevant_docs:
                final_docs.append(doc.model_dump())
            return [json.dumps(final_docs, ensure_ascii=False)], []

        if not relevant_docs:
            return ["没有找到相关的文档来回答这个问题。"], []

        context = [doc.source_code.module_name for doc in relevant_docs]

        # 将 FilterDoc 转化为 SourceCode 方便后续的逻辑继续做处理
        relevant_docs_source: List[SourceCode] = [doc.source_code for doc in relevant_docs]

        # Add relevant docs information
        relevant_docs_info = []
        for doc in relevant_docs_source:
            info = f"- {doc.module_name.replace(self.path, '', 1)}"
            if "original_docs" in doc.metadata:
                original_docs = ", ".join(
                    [
                        doc.replace(self.path, "", 1)
                        for doc in doc.metadata["original_docs"]
                    ]
                )
                info += f" (Original docs: {original_docs})"
            relevant_docs_info.append(info)

        relevant_docs_info = "\n".join(relevant_docs_info)

        first_round_full_docs = []
        second_round_extracted_docs = []
        sencond_round_time = 0

        if self.tokenizer is not None:
            token_limiter = TokenLimiter(
                count_tokens=self.count_tokens,
                full_text_limit=self.full_text_limit,
                segment_limit=self.segment_limit,
                buff_limit=self.buff_limit,
                llm=self.llm,
                args=self.args,
                disable_segment_reorder=self.args.disable_segment_reorder,
            )
            final_relevant_docs = token_limiter.limit_tokens(
                relevant_docs=relevant_docs_source,
                conversations=conversations,
                index_filter_workers=self.args.index_filter_workers or 5,
            )
            first_round_full_docs = token_limiter.first_round_full_docs
            second_round_extracted_docs = token_limiter.second_round_extracted_docs
            sencond_round_time = token_limiter.sencond_round_time

            relevant_docs_source = final_relevant_docs
        else:
            relevant_docs_source = relevant_docs_source[: self.args.index_filter_file_num]

        # Add relevant docs information
        final_relevant_docs_info = []
        for doc in relevant_docs_source:
            info = f"- {doc.module_name.replace(self.path, '', 1)}"
            if "original_docs" in doc.metadata:
                original_docs = ", ".join(
                    [
                        doc.replace(self.path, "", 1)
                        for doc in doc.metadata["original_docs"]
                    ]
                )
                info += f" (Original docs: {original_docs})"
            if "chunk_ranges" in doc.metadata:
                chunk_ranges = json.dumps(
                    doc.metadata["chunk_ranges"], ensure_ascii=False
                )
                info += f" (Chunk ranges: {chunk_ranges})"
            final_relevant_docs_info.append(info)

        final_relevant_docs_info = "\n".join(final_relevant_docs_info)

        printer.print_table_compact(
            data=[
                ["查询", f"{query}"],
                ["相关文档数量", f"{len(relevant_docs)}"],
                ["相关文档列表", f"{relevant_docs_info}"],
                ["仅上下文", f"{str(only_contexts)}"],
                ["过滤时间", f"{filter_time:.2f} 秒"],
                ["最终相关文档数量", f"{len(relevant_docs_source)}"],
                ["第一轮完整文档", f"{len(first_round_full_docs)}"],
                ["第二轮提取文档", f"{len(second_round_extracted_docs)}"],
                ["第二轮时间", f"{sencond_round_time:.2f} 秒"],
                ["最终相关文档列表", f"{final_relevant_docs_info}"],
                ["发送模型 Token", f"{sum([doc.tokens for doc in relevant_docs_source])}"]
            ],
            headers=["Key", "Value"],
            title="查询信息"
        )

        new_conversations = conversations[:-1] + [
            {
                "role": "user",
                "content": self._answer_question.prompt(
                    query=query,
                    relevant_docs=[doc.source_code for doc in relevant_docs_source],
                ),
            }
        ]

        chunks = self.llm.stream_chat_ai(
            conversations=new_conversations, model=self.args.qa_model
        )

        return ((chunk.choices[0].delta.content for chunk in chunks
                if chunk.choices and chunk.choices[0].delta.content),
                context)

    def search_step1(self, conversations):
        """ 文件检索步骤, 混合索引将使用duckdb """
        start_time = time.time()
        query = conversations[-1]["content"]
        source_list = []
        documents = self._retrieve_documents(options={"query": query})
        for d in documents:
            source_list.append(d)
        end_time = time.time() - start_time
        return source_list, end_time

    def search_step2(self, conversations, source_list):
        """ 文件过滤步骤 """
        start_time = time.time()
        relevant_docs: List[FilterDoc] = self.doc_filter.filter_docs(
            conversations=conversations, documents=source_list
        )
        # 过滤 relevant_docs，仅包含 is_relevant=True 的文档
        highly_relevant_docs = [
            doc for doc in relevant_docs if doc.relevance.is_relevant
        ]
        if highly_relevant_docs:
            relevant_docs = highly_relevant_docs
        end_time = time.time() - start_time
        return relevant_docs, end_time

    def search_step3(self, conversations, relevant_docs: List[FilterDoc]):
        """ 文件合并,最终发送到模型的文档数量 """
        start_time = time.time()
        # 将 FilterDoc 转化为 SourceCode 方便后续的逻辑继续做处理
        relevant_docs_source: List[SourceCode] = [doc.source_code for doc in relevant_docs]

        if self.tokenizer is not None:
            token_limiter = TokenLimiter(
                count_tokens=self.count_tokens,
                full_text_limit=self.full_text_limit,
                segment_limit=self.segment_limit,
                buff_limit=self.buff_limit,
                llm=self.llm,
                disable_segment_reorder=self.args.disable_segment_reorder,
            )
            final_relevant_docs = token_limiter.limit_tokens(
                relevant_docs=relevant_docs_source,
                conversations=conversations,
                index_filter_workers=self.args.index_filter_workers or 5,
            )

            relevant_docs_source = final_relevant_docs
        else:
            relevant_docs_source = relevant_docs_source[: self.args.index_filter_file_num]

        end_time = time.time() - start_time
        return relevant_docs_source, end_time

    def search_step4(self, query, conversations, relevant_docs_source):
        """ 最终提问阶段 """
        self.llm.setup_default_model_name("qa_model")
        qa_model = self.llm

        new_conversations = conversations[:-1] + [
            {
                "role": "user",
                "content": self._answer_question.prompt(
                    query=query,
                    relevant_docs=[doc.source_code for doc in relevant_docs_source],
                ),
            }
        ]

        chunks = qa_model.stream_chat_ai(
            conversations=new_conversations
        )

        return ((chunk.choices[0].delta.content for chunk in chunks
                 if chunk.choices and chunk.choices[0].delta.content),
                "")


# Generated from DSLSQL.g4 by ANTLR 4.7.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .DSLSQLParser import DSLSQLParser
else:
    from DSLSQLParser import DSLSQLParser

# This class defines a complete listener for a parse tree produced by DSLSQLParser.
class DSLSQLListener(ParseTreeListener):

    # Enter a parse tree produced by DSLSQLParser#statement.
    def enterStatement(self, ctx:DSLSQLParser.StatementContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#statement.
    def exitStatement(self, ctx:DSLSQLParser.StatementContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#sql.
    def enterSql(self, ctx:DSLSQLParser.SqlContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#sql.
    def exitSql(self, ctx:DSLSQLParser.SqlContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#overwrite.
    def enterOverwrite(self, ctx:DSLSQLParser.OverwriteContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#overwrite.
    def exitOverwrite(self, ctx:DSLSQLParser.OverwriteContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#append.
    def enterAppend(self, ctx:DSLSQLParser.AppendContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#append.
    def exitAppend(self, ctx:DSLSQLParser.AppendContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#errorIfExists.
    def enterErrorIfExists(self, ctx:DSLSQLParser.ErrorIfExistsContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#errorIfExists.
    def exitErrorIfExists(self, ctx:DSLSQLParser.ErrorIfExistsContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#ignore.
    def enterIgnore(self, ctx:DSLSQLParser.IgnoreContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#ignore.
    def exitIgnore(self, ctx:DSLSQLParser.IgnoreContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#booleanExpression.
    def enterBooleanExpression(self, ctx:DSLSQLParser.BooleanExpressionContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#booleanExpression.
    def exitBooleanExpression(self, ctx:DSLSQLParser.BooleanExpressionContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#expression.
    def enterExpression(self, ctx:DSLSQLParser.ExpressionContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#expression.
    def exitExpression(self, ctx:DSLSQLParser.ExpressionContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#ender.
    def enterEnder(self, ctx:DSLSQLParser.EnderContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#ender.
    def exitEnder(self, ctx:DSLSQLParser.EnderContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#format_type.
    def enterFormat_type(self, ctx:DSLSQLParser.Format_typeContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#format_type.
    def exitFormat_type(self, ctx:DSLSQLParser.Format_typeContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#path.
    def enterPath(self, ctx:DSLSQLParser.PathContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#path.
    def exitPath(self, ctx:DSLSQLParser.PathContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#setValue.
    def enterSetValue(self, ctx:DSLSQLParser.SetValueContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#setValue.
    def exitSetValue(self, ctx:DSLSQLParser.SetValueContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#setKey.
    def enterSetKey(self, ctx:DSLSQLParser.SetKeyContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#setKey.
    def exitSetKey(self, ctx:DSLSQLParser.SetKeyContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#db.
    def enterDb(self, ctx:DSLSQLParser.DbContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#db.
    def exitDb(self, ctx:DSLSQLParser.DbContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#tableName.
    def enterTableName(self, ctx:DSLSQLParser.TableNameContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#tableName.
    def exitTableName(self, ctx:DSLSQLParser.TableNameContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#functionName.
    def enterFunctionName(self, ctx:DSLSQLParser.FunctionNameContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#functionName.
    def exitFunctionName(self, ctx:DSLSQLParser.FunctionNameContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#col.
    def enterCol(self, ctx:DSLSQLParser.ColContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#col.
    def exitCol(self, ctx:DSLSQLParser.ColContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#qualifiedName.
    def enterQualifiedName(self, ctx:DSLSQLParser.QualifiedNameContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#qualifiedName.
    def exitQualifiedName(self, ctx:DSLSQLParser.QualifiedNameContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#identifier.
    def enterIdentifier(self, ctx:DSLSQLParser.IdentifierContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#identifier.
    def exitIdentifier(self, ctx:DSLSQLParser.IdentifierContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#strictIdentifier.
    def enterStrictIdentifier(self, ctx:DSLSQLParser.StrictIdentifierContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#strictIdentifier.
    def exitStrictIdentifier(self, ctx:DSLSQLParser.StrictIdentifierContext):
        pass


    # Enter a parse tree produced by DSLSQLParser#quotedIdentifier.
    def enterQuotedIdentifier(self, ctx:DSLSQLParser.QuotedIdentifierContext):
        pass

    # Exit a parse tree produced by DSLSQLParser#quotedIdentifier.
    def exitQuotedIdentifier(self, ctx:DSLSQLParser.QuotedIdentifierContext):
        pass




import os
#from jinja2.runtime import V
from setuptools import find_packages
from setuptools import setup

folder = os.path.dirname(__file__)
version_path = os.path.join(folder, "src", "autocoder_nano", "version.py")

__version__ = ""
with open(version_path) as f:
    exec(f.read(), globals())

req_path = os.path.join(folder, "requirements.txt")
install_requires = []
if os.path.exists(req_path):
    with open(req_path, 'r') as fp:
        install_requires = [line.strip() for line in fp]


readme_path = os.path.join(folder, "README.md")
readme_contents = ""
if os.path.exists(readme_path):
    with open(readme_path, 'r') as fp:
        readme_contents = fp.read().strip()


setup(
    name="autocoder_nano",
    version=__version__,
    description="AutoCoder Nano",
    author="moofs",
    long_description=readme_contents,
    long_description_content_type="text/markdown",
    entry_points={
        'console_scripts': [
            'auto-coder.nano = autocoder_nano.auto_coder_nano:main',
            'auto-coder.bot = autocoder_nano.auto_coder_web:main',
        ],
    },
    package_dir={"": "src"},
    packages=find_packages("src"),
    package_data={
        "autocoder_nano": [
            "data/**/*",
            'app/static/**/*',
            'app/templates/**/*'
        ],
    },
    install_requires=install_requires,
    classifiers=[
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11"
    ],
    python_requires=">=3.10"
)



"""
AutoCoder Nano 终端配色方案
直接定义颜色常量，简单直接使用
"""

# 系统状态类
COLOR_SYSTEM = "bright_blue"          # 系统信息 - 亮蓝色加粗
COLOR_SUCCESS = "bright_green"               # 成功状态 - 绿色加粗
COLOR_ERROR = "bright_red"                   # 错误信息 - 红色加粗
COLOR_WARNING = "bright_yellow"              # 警告信息 - 黄色加粗
COLOR_INFO = "grey50"                   # 一般信息 - 暗白色（低调显示）

# Agent交互类
COLOR_AGENT_START = COLOR_SYSTEM           # Agent启动 - 亮洋红色加粗
COLOR_AGENT_END = COLOR_SUCCESS            # Agent结束 - 亮绿色加粗
COLOR_ITERATION = COLOR_INFO               # 迭代计数 - 亮黄色
COLOR_TOKEN_USAGE = COLOR_INFO             # Token使用 - 亮青色

# 工具相关
COLOR_TOOL_CALL = COLOR_INFO               # 工具调用 - 亮青色加粗
COLOR_TOOL_SUCCESS = COLOR_SUCCESS         # 工具成功 - 亮绿色
COLOR_TOOL_FAILURE = COLOR_ERROR           # 工具失败 - 亮红色

# LLM交互
COLOR_LLM_THINKING = COLOR_INFO            # LLM思考 - 暗白色（低调显示）
COLOR_LLM_OUTPUT = COLOR_INFO              # LLM输出 - 亮白色（清晰显示）
COLOR_LLM_STREAM = COLOR_INFO              # LLM流式输出 - 青色
COLOR_LLM_CALL = COLOR_INFO                # LLM请求 - 暗白色（低调显示）

# 文件操作
COLOR_FILE_READ = COLOR_INFO               # 文件读取 - 亮蓝色
COLOR_FILE_WRITE = COLOR_INFO              # 文件写入 - 亮绿色
COLOR_FILE_CHANGE = COLOR_WARNING          # 文件变更 - 亮黄色

# 特殊状态
COLOR_COMPLETION = COLOR_SUCCESS           # 任务完成 - 亮绿色加粗
COLOR_PROGRESS = COLOR_INFO                # 进度信息 - 亮蓝色
COLOR_DEBUG = COLOR_INFO                   # 调试信息 - 暗白色

# 面板样式
COLOR_PANEL_SUCCESS = COLOR_SUCCESS        # 成功面板边框 - 绿色
COLOR_PANEL_ERROR = COLOR_ERROR            # 错误面板边框 - 红色
COLOR_PANEL_INFO = COLOR_INFO              # 信息面板边框 - 蓝色
COLOR_PANEL_WARNING = COLOR_WARNING        # 警告面板边框 - 黄色
COLOR_PANEL_TOOL = COLOR_INFO              # 工具面板边框 - 青色

# 工具类型颜色映射
TOOL_COLORS = {
    "ReadFileTool": COLOR_FILE_READ,            # 文件读取工具 - 亮蓝色
    "WriteToFileTool": COLOR_FILE_WRITE,        # 文件写入工具 - 亮绿色
    "ReplaceInFileTool": COLOR_FILE_WRITE,      # 文件替换工具 - 亮绿色
    "ExecuteCommandTool": COLOR_SYSTEM,         # 命令执行工具 - 亮蓝色加粗
    "ListFilesTool": COLOR_FILE_READ,           # 文件列表工具 - 亮蓝色
    "SearchFilesTool": COLOR_FILE_READ,         # 文件搜索工具 - 亮蓝色
    "AskFollowupQuestionTool": COLOR_INFO,      # 提问工具 - 亮青色
    "AttemptCompletionTool": COLOR_COMPLETION,  # 完成任务工具 - 亮绿色加粗
    "WebSearchTool": COLOR_SYSTEM,               # 网络搜索工具 - 亮蓝色加粗
    "TodoReadTool": COLOR_SYSTEM,
    "TodoWriteTool": COLOR_SYSTEM
}


def get_tool_color(tool_name):
    """根据工具名称获取对应的颜色"""
    return TOOL_COLORS.get(tool_name, COLOR_TOOL_CALL)  # 默认使用工具调用颜色


import json
import os
# import threading
import time
import platform
from abc import ABC, abstractmethod
from typing import Optional, Dict, Any, Tuple, List

from autocoder_nano.utils.file_utils import generate_file_md5
from autocoder_nano.utils.sys_utils import default_exclude_dirs
from autocoder_nano.actypes import DeleteEvent, AddOrUpdateEvent, SourceCode
from autocoder_nano.rag.doc_loaders import process_file_in_multi_process, process_file_local
from autocoder_nano.utils.printer_utils import Printer


if platform.system() != "Windows":
    import fcntl
else:
    fcntl = None


printer = Printer()


class BaseCacheManager(ABC):
    @abstractmethod
    def get_cache(self, options: Optional[Dict[str, Any]] = None) -> Dict[str, Dict]:
        pass


class AutoCoderRAGAsyncUpdateQueue(BaseCacheManager):
    def __init__(self, path: str, ignore_spec, required_exts: list):
        self.path = path
        self.ignore_spec = ignore_spec
        self.required_exts = required_exts
        self.queue = []

        self.cache_dir = os.path.join(self.path, ".cache")
        self.cache_file = os.path.join(self.cache_dir, "cache.jsonl")
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

        # self.thread = threading.Thread(target=self._process_queue)
        # self.thread.daemon = True
        # self.thread.start()

        self.process_queue()
        self.cache = self.read_cache()

    def get_cache_size(self):
        return len(self.cache.keys())

    def _process_queue(self):
        while 1:
            try:
                self.process_queue()
            except Exception as e:
                printer.print_text(f"Error in process_queue: {e}", style="red")
            time.sleep(600)  # 避免过于频繁的检查

    def load_first(self):
        if self.cache:
            return
        files_to_process = []
        for file_info in self.get_all_files():
            file_path, _, modify_time, file_md5 = file_info
            if file_path not in self.cache or self.cache[file_path].get("md5", "") != file_md5:
                files_to_process.append(file_info)
        if not files_to_process:
            return

        results = []
        for _process_file in files_to_process:
            results.append(process_file_in_multi_process(_process_file))
        for file_info, result in zip(files_to_process, results):
            if result:  # 只有当result不为空时才更新缓存
                self.update_cache(file_info, result)
            else:
                printer.print_text(f"文件 {file_info[0]} 的结果为空，跳过缓存更新", style="yellow")
        self.write_cache()

    def trigger_update(self):
        files_to_process = []
        current_files = set()
        for file_info in self.get_all_files():
            file_path, _, _, file_md5 = file_info
            current_files.add(file_path)
            if file_path not in self.cache or self.cache[file_path].get("md5", "") != file_md5:
                files_to_process.append(file_info)

        deleted_files = set(self.cache.keys()) - current_files
        printer.print_key_value(
            items={"待解析的文件": f"{files_to_process}", "待删除的文件": f"{deleted_files}"},
            title="检查索引更新"
        )
        if deleted_files:
            self.queue.append(DeleteEvent(file_paths=deleted_files))
        if files_to_process:
            self.queue.append(AddOrUpdateEvent(file_infos=files_to_process))

    def process_queue(self):
        while self.queue:
            file_list = self.queue.pop(0)
            if isinstance(file_list, DeleteEvent):
                for item in file_list.file_paths:
                    printer.print_text(f"检测到 {item} 已被移除", style="green")
                    del self.cache[item]
            elif isinstance(file_list, AddOrUpdateEvent):
                for file_info in file_list.file_infos:
                    printer.print_text(f"检测到 {file_info[0]} 已更新", style="green")
                    try:
                        result = process_file_local(file_info[0])
                        if result:  # 只有当result不为空时才更新缓存
                            self.update_cache(file_info, result)
                        else:
                            printer.print_text(f"文件 {file_info[0]} 的结果为空，跳过缓存更新", style="yellow")
                    except Exception as e:
                        printer.print_text(f"SimpleCache 在处理队列时发生错误: {e}", style="red")
            self.write_cache()

    def read_cache(self) -> Dict[str, Dict]:
        cache = {}
        if os.path.exists(self.cache_file):
            with open(self.cache_file, "r") as f:
                for line in f:
                    data = json.loads(line)
                    cache[data["file_path"]] = data
        printer.print_text(f"正在读取文档缓存, 包含 {len(cache.keys())} 个文档", style="green")
        return cache

    def write_cache(self):
        if not fcntl:
            with open(self.cache_file, "w") as f:
                for data in self.cache.values():
                    try:
                        json.dump(data, f, ensure_ascii=False)
                        f.write("\n")
                    except Exception as e:
                        printer.print_text(f"Failed to write {data['file_path']} to .cache/cache.jsonl: {e}",
                                           style="red")
        else:
            lock_file = self.cache_file + ".lock"
            with open(lock_file, "w") as lockf:
                try:
                    # 获取文件锁
                    fcntl.flock(lockf, fcntl.LOCK_EX | fcntl.LOCK_NB)
                    # 写入缓存文件
                    with open(self.cache_file, "w") as f:
                        for data in self.cache.values():
                            try:
                                json.dump(data, f, ensure_ascii=False)
                                f.write("\n")
                            except Exception as e:
                                printer.print_text(f"Failed to write {data['file_path']} to .cache/cache.jsonl: {e}",
                                                   style="red")
                finally:
                    # 释放文件锁
                    fcntl.flock(lockf, fcntl.LOCK_UN)

    def update_cache(self, file_info: Tuple[str, str, float, str], content: List[SourceCode]):
        file_path, relative_path, modify_time, file_md5 = file_info
        self.cache[file_path] = {
            "file_path": file_path,
            "relative_path": relative_path,
            "content": [c.model_dump() for c in content],
            "modify_time": modify_time,
            "md5": file_md5,
        }

    def get_cache(self, options: Optional[Dict[str, Any]] = None):
        self.load_first()
        self.trigger_update()
        # self.process_queue()
        return self.cache

    def build_cache(self):
        pass

    def get_all_files(self) -> List[Tuple[str, str, float, str]]:
        all_files = []
        for root, dirs, files in os.walk(self.path, followlinks=True):
            dirs[:] = [d for d in dirs if not d.startswith(".") and d not in default_exclude_dirs]

            if self.ignore_spec:
                relative_root = os.path.relpath(root, self.path)
                dirs[:] = [
                    d for d in dirs
                    if not self.ignore_spec.match_file(os.path.join(relative_root, d))
                ]
                files[:] = [
                    f for f in files
                    if not self.ignore_spec.match_file(os.path.join(relative_root, f))
                ]

            for file in files:
                if self.required_exts and not any(file.endswith(ext) for ext in self.required_exts):
                    continue
                file_path = os.path.join(root, file)
                relative_path = os.path.relpath(file_path, self.path)
                modify_time = os.path.getmtime(file_path)
                file_md5 = generate_file_md5(file_path)
                all_files.append((file_path, relative_path, modify_time, file_md5))
        return all_files


import typing
from typing import Optional, Union

from autocoder_nano.agent.agentic_edit_tools.base_tool_resolver import BaseToolResolver
from autocoder_nano.agent.agentic_edit_types import ToolResult, AttemptCompletionTool
from autocoder_nano.actypes import AutoCoderArgs

if typing.TYPE_CHECKING:
    from autocoder_nano.agent.agentic_runtime import AgenticRuntime
    from autocoder_nano.agent.agentic_sub import SubAgents


class AttemptCompletionToolResolver(BaseToolResolver):
    def __init__(
            self, agent: Optional[Union['AgenticRuntime', 'SubAgents']],
            tool: AttemptCompletionTool, args: AutoCoderArgs
    ):
        super().__init__(agent, tool, args)
        self.tool: AttemptCompletionTool = tool

    def resolve(self) -> ToolResult:
        """
        Packages the completion result and optional command to signal task completion.
        """
        result_text = self.tool.result
        command = self.tool.command

        # logger.info(f"Resolving AttemptCompletionTool: Result='{result_text[:100]}...', Command='{command}'")

        if not result_text:
            return ToolResult(success=False, message="错误：生成结果不能为空.")

        # The actual presentation of the result happens outside the resolver.
        result_content = {
            "result": result_text,
            "command": command
        }

        # Indicate success in preparing the completion data
        return ToolResult(success=True, message="尝试完成任务.", content=result_content)

    def guide(self) -> str:
        doc = """
        ## attempt_completion（尝试完成任务并输出结果）
        描述：
        - 每次工具使用后，用户会回复该工具使用的结果，即是否成功以及失败原因（如有）。
        - 一旦收到工具使用结果并确认任务完成，使用此工具向用户展示工作成果。
        - 可选地，你可以提供一个 CLI 命令来展示工作成果。用户可能会提供反馈，你可据此进行改进并再次尝试。
        重要提示：
        - 在确认用户已确认之前的工具使用成功之前，不得使用此工具。否则将导致代码损坏和系统故障。
        - 在使用此工具之前，必须在<think></think>标签中自问是否已从用户处确认之前的工具使用成功。如果没有，则不要使用此工具。
        参数：
        - result（必填）：任务的结果，应以最终形式表述，无需用户进一步输入，不得在结果结尾提出问题或提供进一步帮助。
        - command（可选）：用于向用户演示结果的 CLI 命令。
        用法说明：
        <attempt_completion>
        <result>
        成果交付，包括交付物（代码/文档）路径及相关说明
        </result>
        <command>Command to demonstrate result (optional)</command>
        </attempt_completion>
        用法示例：
        场景一：功能开发完成
        目标：已成功添加了一个新功能。
        思维过程：所有开发和测试工作都已完成，现在向用户展示新功能并提供一个命令来验证。
        <attempt_completion>
        <result>
        xxxxx 功能已成功集成到项目中。
        修改文件包括：
        1.src/path/main.py 新增xxx函数完成xxx功能
        2.src/path/run.py 新增xxx函数完成xxx功能
        3.src/README.md  更新文档
        现在您可以使用 python main.py 命令来运行测试，确认新功能的行为。
        </result>
        <command>python main.py</command>
        </attempt_completion>
        """
        return doc



"""
Cache manager for conversation and message caching.

This module provides a high-level interface for managing caches of
conversations and messages, with support for cache warming, invalidation,
and statistics reporting.
"""
from typing import Optional, List, Dict, Any, Callable

from autocoder_nano.context.cache.base_cache import BaseCache
from autocoder_nano.context.cache.memory_cache import MemoryCache
from autocoder_nano.context.models import Conversation, ConversationMessage
from autocoder_nano.utils.printer_utils import Printer


printer = Printer()


class CacheManager:
    """High-level cache manager for conversations and messages."""

    def __init__(
            self,
            conversation_cache: Optional[BaseCache] = None,
            message_cache: Optional[BaseCache] = None
    ):
        """
        Initialize cache manager.

        Args:
            conversation_cache: Cache instance for conversations
            message_cache: Cache instance for messages
        """
        self.conversation_cache = conversation_cache or MemoryCache(
            max_size=100, default_ttl=600.0  # 10 minutes default
        )
        self.message_cache = message_cache or MemoryCache(
            max_size=500, default_ttl=300.0  # 5 minutes default
        )

        # Ensure caches implement required interface
        self._validate_cache_interface(self.conversation_cache)
        self._validate_cache_interface(self.message_cache)

    @staticmethod
    def _validate_cache_interface(cache: BaseCache) -> None:
        """Validate that cache implements required interface."""
        required_methods = ['get', 'set', 'delete', 'clear', 'exists', 'size', 'keys']
        for method in required_methods:
            if not hasattr(cache, method) or not callable(getattr(cache, method)):
                raise TypeError(f"Cache must implement {method} method")

    @staticmethod
    def _get_conversation_key(conversation_id: str) -> str:
        """Generate cache key for conversation."""
        return f"conv:{conversation_id}"

    @staticmethod
    def _get_messages_key(conversation_id: str) -> str:
        """Generate cache key for conversation messages."""
        return f"msgs:{conversation_id}"

    def cache_conversation(
            self,
            conversation: Conversation,
            ttl: Optional[float] = None
    ) -> None:
        """
        Cache a conversation.
        Args:
            conversation: The conversation to cache
            ttl: Time to live in seconds, None for default
        """
        try:
            key = self._get_conversation_key(conversation.conversation_id)
            self.conversation_cache.set(key, conversation, ttl=ttl)
            printer.print_text(f"Cached conversation {conversation.conversation_id}", style="yellow")
        except Exception as e:
            printer.print_text(f"Failed to cache conversation {conversation.conversation_id}: {e}", style="red")

    def get_conversation(self, conversation_id: str) -> Optional[Conversation]:
        """
        Get a conversation from cache.
        Args:
            conversation_id: The conversation ID
        Returns:
            The cached conversation or None if not found
        """
        try:
            key = self._get_conversation_key(conversation_id)
            conversation = self.conversation_cache.get(key)
            if conversation:
                printer.print_text(f"Cache hit for conversation {conversation_id}", style="yellow")
            else:
                printer.print_text(f"Cache miss for conversation {conversation_id}", style="yellow")
            return conversation
        except Exception as e:
            printer.print_text(f"Failed to get conversation {conversation_id} from cache: {e}", style="red")
            return None

    def cache_messages(
            self,
            conversation_id: str,
            messages: List[ConversationMessage],
            ttl: Optional[float] = None
    ) -> None:
        """
        Cache messages for a conversation.
        Args:
            conversation_id: The conversation ID
            messages: List of messages to cache
            ttl: Time to live in seconds, None for default
        """
        try:
            key = self._get_messages_key(conversation_id)
            self.message_cache.set(key, messages, ttl=ttl)
            printer.print_text(f"Cached {len(messages)} messages for conversation {conversation_id}", style="yellow")
        except Exception as e:
            printer.print_text(f"Failed to cache messages for conversation {conversation_id}: {e}", style="red")

    def get_messages(self, conversation_id: str) -> Optional[List[ConversationMessage]]:
        """
        Get messages from cache.
        Args:
            conversation_id: The conversation ID
        Returns:
            List of cached messages or None if not found
        """
        try:
            key = self._get_messages_key(conversation_id)
            messages = self.message_cache.get(key)
            if messages:
                printer.print_text(f"Cache hit for messages of conversation {conversation_id}", style="yellow")
            else:
                printer.print_text(f"Cache miss for messages of conversation {conversation_id}", style="yellow")
            return messages
        except Exception as e:
            printer.print_text(f"Failed to get messages for conversation {conversation_id} from cache: {e}",
                               style="red")
            return None

    def invalidate_conversation(self, conversation_id: str) -> bool:
        """
        Invalidate cached conversation.
        Args:
            conversation_id: The conversation ID
        Returns:
            True if conversation was cached and removed, False otherwise
        """
        try:
            key = self._get_conversation_key(conversation_id)
            result = self.conversation_cache.delete(key)
            if result:
                printer.print_text(f"Invalidated conversation {conversation_id}", style="yellow")
            return result
        except Exception as e:
            printer.print_text(f"Failed to invalidate conversation {conversation_id}: {e}", style="red")
            return False

    def invalidate_messages(self, conversation_id: str) -> bool:
        """
        Invalidate cached messages.
        Args:
            conversation_id: The conversation ID
        Returns:
            True if messages were cached and removed, False otherwise
        """
        try:
            key = self._get_messages_key(conversation_id)
            result = self.message_cache.delete(key)
            if result:
                printer.print_text(f"Invalidated messages for conversation {conversation_id}", style="yellow")
            return result
        except Exception as e:
            printer.print_text(f"Failed to invalidate messages for conversation {conversation_id}: {e}", style="red")
            return False

    def invalidate_all(self, conversation_id: str) -> Dict[str, bool]:
        """
        Invalidate all cached data for a conversation.
        Args:
            conversation_id: The conversation ID
        Returns:
            Dictionary with invalidation results
        """
        return {
            "conversation": self.invalidate_conversation(conversation_id),
            "messages": self.invalidate_messages(conversation_id)
        }

    def warm_conversation_cache(
            self,
            data_loader: Callable[[], List[Conversation]]
    ) -> int:
        """
        Warm conversation cache with data.
        Args:
            data_loader: Function that returns conversations to cache
        Returns:
            Number of conversations cached
        """
        try:
            conversations = data_loader()
            count = 0

            for conversation in conversations:
                self.cache_conversation(conversation)
                count += 1

            printer.print_text(f"Warmed conversation cache with {count} conversations", style="green")
            return count

        except Exception as e:
            printer.print_text(f"Failed to warm conversation cache: {e}", style="red")
            return 0

    def cache_conversations(
            self,
            conversations: List[Conversation],
            ttl: Optional[float] = None
    ) -> int:
        """
        Cache multiple conversations.
        Args:
            conversations: List of conversations to cache
            ttl: Time to live in seconds, None for default
        Returns:
            Number of conversations successfully cached
        """
        count = 0
        for conversation in conversations:
            try:
                self.cache_conversation(conversation, ttl=ttl)
                count += 1
            except Exception as e:
                printer.print_text(f"Failed to cache conversation {conversation.conversation_id}: {e}", style="red")

        return count

    def invalidate_conversations(self, conversation_ids: List[str]) -> Dict[str, bool]:
        """
        Invalidate multiple conversations.
        Args:
            conversation_ids: List of conversation IDs to invalidate
        Returns:
            Dictionary mapping conversation IDs to invalidation results
        """
        results = {}
        for conversation_id in conversation_ids:
            results[conversation_id] = self.invalidate_conversation(conversation_id)

        return results

    def clear_all_caches(self) -> None:
        """Clear all caches."""
        try:
            self.conversation_cache.clear()
            self.message_cache.clear()
            printer.print_text("Cleared all caches", style="green")
        except Exception as e:
            printer.print_text(f"Failed to clear caches: {e}", style="red")

    def get_cache_statistics(self) -> Dict[str, Any]:
        """
        Get statistics for all caches.
        Returns:
            Dictionary with cache statistics
        """
        try:
            stats = {
                "conversation_cache": {
                    "size": self.conversation_cache.size(),
                    "max_size": getattr(self.conversation_cache, 'max_size', 'unknown')
                },
                "message_cache": {
                    "size": self.message_cache.size(),
                    "max_size": getattr(self.message_cache, 'max_size', 'unknown')
                }
            }

            # Add detailed stats if available
            if hasattr(self.conversation_cache, 'get_statistics'):
                stats["conversation_cache"].update(
                    self.conversation_cache.get_statistics()
                )

            if hasattr(self.message_cache, 'get_statistics'):
                stats["message_cache"].update(
                    self.message_cache.get_statistics()
                )

            return stats

        except Exception as e:
            printer.print_text(f"Failed to get cache statistics: {e}", style="red")
            return {
                "conversation_cache": {"size": 0, "max_size": "unknown"},
                "message_cache": {"size": 0, "max_size": "unknown"},
                "error": str(e)
            }

    def is_conversation_cached(self, conversation_id: str) -> bool:
        """
        Check if a conversation is cached.
        Args:
            conversation_id: The conversation ID
        Returns:
            True if conversation is cached, False otherwise
        """
        try:
            key = self._get_conversation_key(conversation_id)
            return self.conversation_cache.exists(key)
        except Exception as e:
            printer.print_text(f"Failed to check if conversation {conversation_id} is cached: {e}", style="red")
            return False

    def is_messages_cached(self, conversation_id: str) -> bool:
        """
        Check if messages are cached.

        Args:
            conversation_id: The conversation ID

        Returns:
            True if messages are cached, False otherwise
        """
        try:
            key = self._get_messages_key(conversation_id)
            return self.message_cache.exists(key)
        except Exception as e:
            printer.print_text(f"Failed to check if messages for conversation {conversation_id} are cached: {e}",
                               style="red")
            return False

    def get_cached_conversation_ids(self) -> List[str]:
        """
        Get all cached conversation IDs.

        Returns:
            List of conversation IDs currently cached
        """
        try:
            keys = self.conversation_cache.keys()
            # Extract conversation IDs from cache keys
            conversation_ids = []
            for key in keys:
                if key.startswith("conv:"):
                    conversation_ids.append(key[5:])  # Remove "conv:" prefix
            return conversation_ids
        except Exception as e:
            printer.print_text(f"Failed to get cached conversation IDs: {e}", style="red")
            return []